@@ -1,6 +1,6 @@
-// { dg-options " -std=gnu++11 " }
+// { dg-do run { target c++11 } }
 
-// Copyright (C) 2014-2016 Free Software Foundation, Inc.
+// Copyright (C) 2014-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the terms
@@ -27,12 +27,10 @@
 void
 test01()
 {
-  bool test __attribute__((unused)) = true;
-
   int i = 0;
   void* ptr = &i;
   auto space = sizeof(i);
-  auto p2 = std::align(alignof(i), space, ptr, space);
+  auto p2 = std::align(alignof(int), space, ptr, space);
   VERIFY( ptr == &i );
   VERIFY( p2 == &i );
   VERIFY(space == sizeof(i));
