@@ -1,7 +1,4 @@
-// Derived from libstdc++/12048 by LJR <ljrittle@acm.org> with
-// reminder from Petur Runolfsson <peturr02@ru.is>.
-
-// Copyright (C) 2003-2015 Free Software Foundation, Inc.
+// Copyright (C) 2003-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -20,6 +17,7 @@
 
 #include <iostream>
 #include <cstdio>
+#include <cwchar>
 #include <testsuite_hooks.h>
 
 void
@@ -28,12 +26,12 @@ test01()
   VERIFY( std::freopen("cin_unget-1.txt", "r", stdin) );
 
   wchar_t c1;
-  wchar_t c2;
+  std::wint_t c2;
   std::wcin.get(c1);
   std::wcin.unget();
-  std::wcin.get(c2);
   VERIFY( std::wcin.good() );
-  VERIFY( c1 == c2 );
+  c2 = std::fgetwc(stdin);
+  VERIFY( c2 == std::char_traits<wchar_t>::to_int_type(c1) );
 }
 
 int main(void)
