@@ -1,6 +1,6 @@
-// 2003-04-30  Petur Runolfsson <peturr02@ru.is>
+// 2004-01-11  Petur Runolfsson  <peturr02@ru.is>
 
-// Copyright (C) 2003-2017 Free Software Foundation, Inc.
+// Copyright (C) 2004-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -17,29 +17,43 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-#include <testsuite_hooks.h>
-#include <cstdio>
 #include <iostream>
-#include <cwchar>
+#include <string>
+#include <locale>
 
-void test02()
+// libstdc++/13582
+void test01()
 {
   using namespace std;
 
-  wint_t c1 = fgetwc(stdin);
-  wint_t c2 = wcin.rdbuf()->sputbackc(c1);
-  VERIFY( c2 == c1 );
+  ios_base::sync_with_stdio(false);
+  wcout << "Type in 12345\n";
   
-  wint_t c3 = fgetwc(stdin);
-  VERIFY( c3 == c1 );
-  ungetwc(c3, stdin);
+  wstring str;
+  wchar_t c;
   
-  wint_t c4 = wcin.rdbuf()->sgetc();
-  VERIFY( c4 == c3 );
+  if (wcin.get(c) && !isspace(c, wcin.getloc()))
+    {
+      str.push_back(c);
+      wcin.imbue(locale(ISO_8859(1,en_US)));
+    }
+
+  if (wcin.get(c) && !isspace(c, wcin.getloc()))
+    {
+      str.push_back(c);
+      wcin.imbue(locale(ISO_8859(15,fr_FR)));
+    }
+
+  while (wcin.get(c) && !isspace(c, wcin.getloc()))
+    {
+      str.push_back(c);
+    }
+  
+  wcout << str << endl;
 }
 
 int main()
 {
-  test02();
+  test01();
   return 0;
 }
