@@ -1,9 +1,9 @@
-// { dg-options "-std=gnu++11" }
+// { dg-do run { target c++11 } }
 
 //
 // 2010-06-16  Stephen M. Webb <stephen.webb@bregmasoft.ca>
 //
-// Copyright (C) 2010-2015 Free Software Foundation, Inc.
+// Copyright (C) 2010-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -29,16 +29,14 @@
 void
 test01()
 {
-  bool test __attribute__((unused)) = true;
-
-	try
-	{
-		std::regex  re("a\\{1,2,3\\}", std::regex::basic);
-	}
-	catch (std::regex_error& ex)
-	{
-		VERIFY( ex.code() == std::regex_constants::error_brace );
-	}
+  try
+    {
+      std::regex  re("a\\{1,2,3\\}", std::regex::basic);
+    }
+  catch (std::regex_error& ex)
+    {
+      VERIFY( ex.code() == std::regex_constants::error_brace );
+    }
 }
 
 
