@@ -1,6 +1,6 @@
 // 2003-02-26 Benjamin Kosnik <bkoz@redhat.com>
 
-// Copyright (C) 2003-2015 Free Software Foundation, Inc.
+// Copyright (C) 2003-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -21,12 +21,13 @@
 
 #include <testsuite_hooks.h>
 
-// c++/3111
+// libstdc++/7986
 int main()
 {
   using namespace __gnu_test;
 
-  verify_demangle("_Z1fKPFiiE", "f(int (* const)(int))");
+  // cplus-dem FAIL  
+  verify_demangle("_Z1jM1AFivEPS1_", "j(int (A::*)(), int (A::**)())");
 
   return 0;
 }
